# Generated by roxygen2: do not edit by hand

S3method(AIC,UComp)
S3method(BIC,UComp)
S3method(coef,UComp)
S3method(fitted,ETS)
S3method(fitted,TETS)
S3method(fitted,UComp)
S3method(logLik,UComp)
S3method(plot,ARIMA)
S3method(plot,ETS)
S3method(plot,TETS)
S3method(plot,UComp)
S3method(predict,UComp)
S3method(print,ARIMA)
S3method(print,ETS)
S3method(print,TETS)
S3method(print,UComp)
S3method(residuals,ARIMA)
S3method(residuals,ETS)
S3method(residuals,TETS)
S3method(residuals,UComp)
S3method(summary,ARIMA)
S3method(summary,ETS)
S3method(summary,TETS)
S3method(summary,UComp)
S3method(tsdiag,UComp)
export(ARIMA)
export(ARIMAestim)
export(ARIMAforecast)
export(ARIMAsetup)
export(ARIMAvalidate)
export(Accuracy)
export(DAYforecast)
export(ETS)
export(ETScomponents)
export(ETSestim)
export(ETSforecast)
export(ETSsetup)
export(ETSvalidate)
export(TETS)
export(TETScomponents)
export(TETSestim)
export(TETSforecast)
export(TETSsetup)
export(TETSvalidate)
export(UC)
export(UCcommand)
export(UCcomponents)
export(UCdisturb)
export(UCestim)
export(UCfilter)
export(UCforecast)
export(UChp)
export(UCsetup)
export(UCsmooth)
export(UCvalidate)
export(WEEKforecast)
export(acft)
export(arma2tsi)
export(armaFilter)
export(auxInvBoxCox)
export(box.cox)
export(colMedians)
export(conv)
export(cusum)
export(dif)
export(extract)
export(gaussTest)
export(getp0)
export(ident)
export(inv.box.cox)
export(invBoxCox)
export(plotAcfPacf)
export(plotBar)
export(plotSlide)
export(plus_one)
export(removeNaNs)
export(roots)
export(rowMedians)
export(size)
export(slide)
export(slideAux)
export(sumStats)
export(tests)
export(tsDisplay)
export(varTest)
export(zplane)
useDynLib(UComp, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(stats)
import(utils)
import(tsoutliers)
import(ggplot2)
import(gridExtra)
import(tsibble)
import(parallel)
import(ggforce)
