# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

UCompC <- function(commands, ys, us, models, hs, lambdas, outliers, tTests, criterions, periodss, rhoss, verboses, stepwises, p0s, armas, TVPs, seass, trendOptionss, seasonalOptionss, irregularOptionss) {
    .Call(`_UComp_UCompC`, commands, ys, us, models, hs, lambdas, outliers, tTests, criterions, periodss, rhoss, verboses, stepwises, p0s, armas, TVPs, seass, trendOptionss, seasonalOptionss, irregularOptionss)
}

ETSc <- function(commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s) {
    .Call(`_UComp_ETSc`, commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s)
}

ARIMAc <- function(commands, ys, us, orderss, cnsts, ss, criterions, hs, verboses, lambdas, maxOrderss, bootstraps, nSimuls, fasts, identDiffs, identMethods) {
    .Call(`_UComp_ARIMAc`, commands, ys, us, orderss, cnsts, ss, criterions, hs, verboses, lambdas, maxOrderss, bootstraps, nSimuls, fasts, identDiffs, identMethods)
}

TETSc <- function(commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s, Ymins, Ymaxs) {
    .Call(`_UComp_TETSc`, commands, ys, us, models, ss, hs, criterions, armaIdents, identAlls, forIntervalss, bootstraps, nSimuls, verboses, lambdas, alphaLs, betaLs, gammaLs, phiLs, p0s, Ymins, Ymaxs)
}

