% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{rowMedians}
\alias{rowMedians}
\title{rowMedians}
\usage{
rowMedians(x, na.rm = TRUE, ...)
}
\arguments{
\item{x}{a matrix}

\item{na.rm}{boolean indicating whether to remove nans}

\item{...}{rest of inputs}
}
\value{
A vector with all the medians in rows
}
\description{
Medians of matrix by rows
}
\examples{
s <- rowMedians(matrix(4, 3, 2))
}
\seealso{
\code{\link{colMedians}}, \code{\link{tests}},
         \code{\link{sumStats}}, \code{\link{gaussTest}}, \code{\link{ident}},
         \code{\link{cusum}}, \code{\link{varTest}}, \code{\link{conv}},
         \code{\link{armaFilter}}, \code{\link{dif}}, \code{\link{roots}},
         \code{\link{zplane}}, \code{\link{acft}}, \code{\link{slide}},
         \code{\link{plotSlide}}, \code{\link{Accuracy}}, \code{\link{tsDisplay}},
         \code{\link{size}}
}
\author{
Diego J. Pedregal
}
