% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ch4}
\alias{ch4}
\title{Methane concentration at Cape Grim in Australia}
\format{
Time series objects.
\describe{
  Monthly data from January 1992 to December 2019
}
}
\source{
\href{https://www.csiro.au/en/research/natural-environment/atmosphere/Latest-greenhouse-gas-data}{CH4 data}
}
\usage{
ch4
}
\value{
No return value, called for side effects
}
\description{
Methane concentration at Cape Grim in Australia (ch4).
}
\examples{
\donttest{
 ch4
}
}
\keyword{datasets}
