% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ARIMAvalidate.R
\name{ARIMAvalidate}
\alias{ARIMAvalidate}
\title{ARIMAvalidate}
\usage{
ARIMAvalidate(m)
}
\arguments{
\item{m}{an object of type \code{ARIMA} created with \code{ARIMAforecast}}
}
\value{
The same input object with the appropriate fields 
filled in, in particular:
\item{table}{Estimation and validation table}
}
\description{
Shows a table of estimation and diagnostics results for ARIMA models
}
\examples{
\donttest{
m1 <- ARIMAforecast(log(gdp))
m1 <- ARIMAvalidate(m1)
}
}
\seealso{
\code{\link{ARIMA}}, \code{\link{ARIMAforecast}}, \code{\link{ARIMAvalidate}},
}
\author{
Diego J. Pedregal
}
